@@ -1,4 +1,4 @@
-from typing import List, Optional
+from typing import Optional
 
 import strawberry
 from strawberry import UNSET
@@ -11,6 +11,6 @@ from .DatasetExampleInput import DatasetExampleInput
 @strawberry.input
 class AddExamplesToDatasetInput:
     dataset_id: GlobalID
-    examples: List[DatasetExampleInput]
+    examples: list[DatasetExampleInput]
     dataset_version_description: Optional[str] = UNSET
     dataset_version_metadata: Optional[JSON] = UNSET
